/**
 * Sample code for "Untestable Code" serie. Find more on
 * http://kaczanowscy.pl/tomek.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including * without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package pl.kaczanowscy.uc.domain;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

import junit.framework.TestCase;
import pl.kaczanowscy.uc.domain.EntityX;
import pl.kaczanowscy.uc.domain.InvalidItemStatus;
import pl.kaczanowscy.uc.domain.ServiceA;

/**
 * @author Tomek Kaczanowski
 * 
 */
public class ServiceATest extends TestCase {

	private static final BigDecimal TOTAL = BigDecimal.TEN;

	class MyServiceA extends ServiceA {
		private boolean saved = false;

		@Override
		BigDecimal computeTotal(List items) throws InvalidItemStatus {
			return TOTAL;
		}

		@Override
		List find(EntityX data) {
			return Collections.EMPTY_LIST;
		}

		@Override
		void save(EntityX data) {
			this.saved = true;
		}

		public boolean wasSaved() {
			return saved;
		}

	};

	/** Test spy and SUT at the same time. */
	private MyServiceA sA = new MyServiceA();

	public void testBusinessOperation() throws InvalidItemStatus {
		EntityX entity = new EntityX(1, "abc", "def");
		sA.doBusinessOperationXyz(entity);
		assertEquals(TOTAL, entity.getTotal());
		assertTrue(sA.wasSaved());
	}
}
